/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "app-icon-provider.h"

#include <QDebug>
#include <QIcon>
#include <QFile>
#include <qt5xdg/XdgIcon>

namespace UkuiMenu {

QSize AppIconProvider::s_defaultSize = QSize(128, 128);

AppIconProvider::AppIconProvider() : QQuickImageProvider(QQmlImageProviderBase::Pixmap)
{

}

QPixmap AppIconProvider::requestPixmap(const QString &id, QSize *size, const QSize &requestedSize)
{
    return AppIconProvider::getPixmap(id, size, requestedSize);
}

QPixmap AppIconProvider::getPixmap(const QString &id, QSize *size, const QSize &requestedSize)
{
    // 不处理sourceSize请求
    Q_UNUSED(requestedSize);
    QPixmap pixmap;
    loadPixmap(id, pixmap);

    if (size) {
        QSize pixmapSize = pixmap.size();
        size->setWidth(pixmapSize.width());
        size->setHeight(pixmapSize.height());
    }

    return pixmap;
}

void AppIconProvider::loadPixmap(const QString &id, QPixmap &pixmap)
{
    if (id.isEmpty()) {
        loadDefault(pixmap);
        return;
    }

    bool isOk;
    QUrl url(id);
    if (!url.scheme().isEmpty()) {
        isOk = loadPixmapFromUrl(url, pixmap);

    } else if (id.startsWith(QLatin1String("/")) || id.startsWith(QLatin1String(":/"))) {
        //qrc path: :/xxx/xxx.png
        isOk = loadPixmapFromPath(id, pixmap);

    } else {
        isOk = loadPixmapFromTheme(id, pixmap);
        if (!isOk) {
            isOk = loadPixmapFromXdg(id, pixmap);
        }
    }

    if (!isOk) {
        loadDefault(pixmap);
    }
}

// see: https://doc.qt.io/archives/qt-5.12/qurl.html#details
bool AppIconProvider::loadPixmapFromUrl(const QUrl &url, QPixmap &pixmap)
{
    QString path = url.path();
    if (path.isEmpty()) {
        qWarning() << "Error: loadPixmapFromUrl, path is empty.";
        return false;
    }

    // qrc example: the Path ":/images/cut.png" or the URL "qrc:///images/cut.png"
    // see: https://doc.qt.io/archives/qt-5.12/resources.html
    if (url.scheme() == QLatin1String("qrc")) {
        path.prepend(QLatin1String(":"));
        return loadPixmapFromPath(path, pixmap);
    }

    // net?
    return false;
}

bool AppIconProvider::loadPixmapFromPath(const QString &path, QPixmap &pixmap)
{
    if (!QFile::exists(path)) {
        qWarning() << "Error: loadPixmapFromPath, File dose not exists." << path;
        return false;
    }

    QPixmap filePixmap;
    bool isOk = filePixmap.load(path);
    if (!isOk) {
        return false;
    }

    pixmap.swap(filePixmap);

    return true;
}

bool AppIconProvider::loadPixmapFromTheme(const QString &name, QPixmap &pixmap)
{
    if (!QIcon::hasThemeIcon(name)) {
        return false;
    }
    QIcon icon = QIcon::fromTheme(name);
    pixmap = icon.pixmap(AppIconProvider::s_defaultSize);
    return true;
}

void AppIconProvider::loadDefault(QPixmap &pixmap)
{
    if (!loadPixmapFromTheme("application-x-desktop", pixmap)) {
        loadPixmapFromPath(":/res/icon/application-x-desktop.png", pixmap);
    }
}

bool AppIconProvider::loadPixmapFromXdg(const QString &name, QPixmap &pixmap)
{
    QIcon icon = XdgIcon::fromTheme(name);
    if (icon.isNull()) {
        qWarning() << "Error: loadPixmapFromXdg, icon dose not exists. name:" << name;
        return false;
    }

    pixmap = icon.pixmap(AppIconProvider::s_defaultSize);
    return true;
}

} // UkuiMenu
