/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "menu-extension.h"
#include "extensions/favorite-extension.h"

#include <utility>
#include <QDir>
#include <QPluginLoader>
#include <QDebug>

namespace UkuiMenu {

MenuExtension *MenuExtension::instance()
{
    static MenuExtension menuExtension;
    return &menuExtension;
}

MenuExtension::MenuExtension()
{
    qRegisterMetaType<ExtensionModel*>("ExtensionModel*");

    // TODO load extension from filesystem.
    loadExtensions();
    // register extension.
    registerExtension(new FavoriteExtension(this));

    initModel();
}

void MenuExtension::registerExtension(MenuExtensionIFace *extension)
{
    if (!extension) {
        return;
    }

    extension->setParent(this);
    if (m_extensions.contains(extension->name())) {
        return;
    }

    m_extensions.insert(extension->name(), extension);
}

ExtensionModel *MenuExtension::extensionModel()
{
    return m_model;
}

void MenuExtension::initModel()
{
    QVector<MenuExtensionIFace*> data;

    QMap<QString, MenuExtensionIFace*>::const_iterator iterator = m_extensions.constBegin();
    for (; iterator != m_extensions.constEnd(); ++iterator) {
        MenuExtensionIFace* extension = iterator.value();
        data.append(extension);
    }

    std::sort(data.begin(), data.end(), [](MenuExtensionIFace* a, MenuExtensionIFace* b) {
        return a->index() <= b->index();
    });

    m_model = new ExtensionModel(data, this);
    if (!m_model) {
        qWarning() << "MenuExtension: unable to init the model of menu-extension.";
    }
}

void MenuExtension::loadExtensions()
{
    QDir pluginsDir(UKUI_MENU_EXTENSION_DIR);
    pluginsDir.setFilter(QDir::Files);
    for(const QString& fileName : pluginsDir.entryList(QDir::Files)) {
        QPluginLoader pluginLoader(pluginsDir.absoluteFilePath(fileName));
        QJsonObject metaData = pluginLoader.metaData().value("MetaData").toObject();
        QString type = metaData.value("Type").toString();
        QString version = metaData.value("Version").toString();
        if(type != UKUI_MENU_EXTENSION_TYPE) {
            continue;
        }

        if(version != UKUI_MENU_EXTENSION_IFACE_VERSION) {
            qWarning() << "UKUI_MENU_EXTENSION version check failed:" << fileName << "version:" << version << "iface version : " << UKUI_MENU_EXTENSION_IFACE_VERSION;
            continue;
        }

        QObject *obj = pluginLoader.instance();
        if (!obj) {
            continue;
        }

        MenuExtensionIFace *plugin = qobject_cast<MenuExtensionIFace*>(obj);
        if (!plugin) {
            continue;
        }

        registerExtension(plugin);
    }
}

ExtensionModel::ExtensionModel(const QVector<MenuExtensionIFace*> &extensions, QObject *parent)
    : QAbstractListModel(parent), m_extensions(extensions)
{
    m_roleNames.insert(Name, "name");
    m_roleNames.insert(Url, "url");
    m_roleNames.insert(Data, "data");

    for (int i = 0; i < m_extensions.size(); ++i) {
        connect(m_extensions.at(i), &MenuExtensionIFace::dataUpdated, this, [this, i] {
            extensionDataChanged(i);
        });
    }
}

int ExtensionModel::rowCount(const QModelIndex &parent) const
{
    return m_extensions.size();
}

QVariant ExtensionModel::data(const QModelIndex &index, int role) const
{
    int row = index.row();

    if (row < 0 || row >= m_extensions.size()) {
        return {};
    }

    switch (role) {
        case Name:
            return m_extensions.at(row)->name();
        case Url:
            return m_extensions.at(row)->url();
        case Data: {
            return m_extensions.at(row)->data();
        }
        default:
            break;
    }

    return {};
}

QHash<int, QByteArray> ExtensionModel::roleNames() const
{
    return m_roleNames;
}

void ExtensionModel::send(int index, QVariantMap data)
{
    if (index < 0 || index >= m_extensions.size()) {
        return;
    }

    m_extensions.at(index)->receive(std::move(data));
}

void ExtensionModel::extensionDataChanged(int index)
{
    if (index < 0 || index >= m_extensions.size()) {
        return;
    }

    QVector<int> roles(1, Data);
    QModelIndex modelIndex = createIndex(index, 0);
    Q_EMIT dataChanged(modelIndex, modelIndex, roles);
}

} // UkuiMenu
