/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "menu-manager.h"
#include "app-manager.h"
#include "app-data-manager.h"

#include <QMenu>
#include <QCursor>
#include <QDBusInterface>
#include <QDBusReply>
#include <QSettings>
#include <QFile>
#include <QDebug>
#include <QStandardPaths>
#include <QFileInfo>
#include <QProcess>
#include <QDir>
#include <QPluginLoader>
#include <QJsonObject>

using namespace UkuiMenu;

class AppMenuProvider : public MenuProvider
{
public:
    AppMenuProvider();
    int index() override { return 1024; }
    bool isSupport(const RequestType &type) override;
    QList<QAction *> generateActions(QObject *parent, const QVariant &data, const MenuInfo::Location &location, const QString &locationId) override;

private:
    void addToTop(QObject *parent, const QString &appId, const int &appTop, QList<QAction *> &list);
    void addToPanelAction(QObject *parent, const QString &appId, QList<QAction *> &list);
    void addToDesktopAction(QObject *parent, const QString &appId, QList<QAction *> &list);
    void addUninstall(QObject *parent, const QString &appId, QList<QAction *> &list);
private:
    QStringList m_systemApps;
};

AppMenuProvider::AppMenuProvider()
{
    if (QFile::exists(UKUI_MENU_GLOBAL_CONFIG_FILE)) {
        QSettings settings(UKUI_MENU_GLOBAL_CONFIG_FILE, QSettings::IniFormat);
        settings.beginGroup("System Apps");
        m_systemApps.append(settings.allKeys());
        settings.endGroup();
    }
}

bool AppMenuProvider::isSupport(const MenuProvider::RequestType &type)
{
    return type == DataType;
}

QList<QAction *>
AppMenuProvider::generateActions(QObject *parent, const QVariant &data, const MenuInfo::Location &location,
                                 const QString &locationId)
{
    if (!parent) {
        return {};
    }

    DataEntity app = data.value<DataEntity>();
    if (app.type() != DataType::Normal) {
        return {};
    }

    QList<QAction *> list;
    QString appId = app.id();
    int appTop = app.top();

    switch (location) {
        case MenuInfo::AppList: {
            //置顶
            if (locationId == "all") {
                addToTop(parent, appId, appTop, list);
            }
        }
        case MenuInfo::Extension:
        case MenuInfo::FolderPage:
        case MenuInfo::FullScreen:
        case MenuInfo::FullScreenFolder:
            // 卸载
            addToPanelAction(parent, appId, list);
            // 添加到任务栏
            addUninstall(parent, appId, list);
            //添加到桌面快捷方式
            addToDesktopAction(parent, appId, list);
            break;
        default:
            break;
    }

    return list;
}

void AppMenuProvider::addToTop(QObject *parent, const QString &appId, const int &appTop, QList<QAction *> &list)
{
    QString actionName = (appTop == 0) ? QObject::tr("Fixed to all applications") : QObject::tr("Unfixed from all applications");
    list << new QAction(actionName, parent);
    QObject::connect(list.last(), &QAction::triggered, parent, [appId, appTop] {
        AppDataManager::instance()->fixToTop(appId, appTop);
    });
}

void AppMenuProvider::addToPanelAction(QObject *parent, const QString &appId, QList<QAction *> &list)
{
    QDBusInterface iface("com.ukui.panel.desktop", "/", "com.ukui.panel.desktop", QDBusConnection::sessionBus());

    if (!iface.isValid()) {
        qWarning() << "Panel desktop dbusinterface error";
        return;
    }

    iface.setTimeout(1);
    QDBusReply<bool> isFixedOnTaskBar = iface.call("CheckIfExist", appId);

    if (!isFixedOnTaskBar.isValid()) {
        qWarning() << "Panel desktop dbusinterface call CheckIfExist timeout";
        return;
    }

    QString actionName = isFixedOnTaskBar ? QObject::tr("Remove from taskbar") : QObject::tr("Add to taskbar");
    QString functionName = isFixedOnTaskBar ? "RemoveFromTaskbar" : "AddToTaskbar";
    list << new QAction(actionName, parent);

    QObject::connect(list.last(), &QAction::triggered, parent, [appId, functionName] {
        QDBusInterface iface("com.ukui.panel.desktop", "/", "com.ukui.panel.desktop", QDBusConnection::sessionBus());

        if (!iface.isValid()) {
            qWarning() << "Panel desktop dbusinterface error";
            return;
        }

        iface.setTimeout(1);
        QDBusReply<bool> ret = iface.call(functionName, appId);

        if (!ret.isValid() || !ret) {
            qWarning() << "Add/Remove from taskbar error";
        }
    });
}

void AppMenuProvider::addToDesktopAction(QObject *parent, const QString &appId, QList<QAction *> &list)
{
    list << new QAction(QObject::tr("Add to desktop shortcuts"), parent);

    QString desktopPath = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
    QString path = QString(desktopPath + "/" + QFileInfo(appId).fileName());

    if (QFile::exists(path)) {
        list.last()->setEnabled(false);
        return;
    }

    QObject::connect(list.last(), &QAction::triggered, parent, [appId, path] {
        QFile file(appId);
        bool ret = file.copy(path);
        if (ret) {
            QProcess::startDetached("chmod a+x " + path);
        }
    });
}

void AppMenuProvider::addUninstall(QObject *parent, const QString &appId, QList<QAction *> &list)
{
    bool isSystemApp = std::any_of(m_systemApps.constBegin(), m_systemApps.constEnd(), [&appId](const QString &appName) {
        return appId.contains(appName);
    });
    if (!isSystemApp) {
        list << new QAction(QObject::tr("Uninstall"), parent); //QIcon::fromTheme("edit-delete-symbolic")
        QObject::connect(list.last(), &QAction::triggered, parent, [appId] {
            AppManager::instance()->launchBinaryApp("kylin-uninstaller", appId);
        });
    }
}

MenuManager *MenuManager::instance()
{
    static MenuManager manager;
    return &manager;
}

class MenuManagerPrivate
{
public:
    QList<MenuProvider*> providers;
};

MenuManager::MenuManager() : d(new MenuManagerPrivate)
{
    registerMenuProvider(new AppMenuProvider);
    loadMenus();
}

MenuManager::~MenuManager()
{
    for (auto &provider : d->providers) {
        delete provider;
        provider = nullptr;
    }
}

void MenuManager::registerMenuProvider(MenuProvider *provider)
{
    if (!provider) {
        return;
    }

    d->providers.append(provider);

    std::sort(d->providers.begin(), d->providers.end(), [=] (MenuProvider *a, MenuProvider *b) {
        return a->index() < b->index();
    });
}

void MenuManager::showMenu(const QString &appid, const MenuInfo::Location location, const QString &lid, const QPoint &point)
{
    DataEntity app;
    if (AppDataManager::instance()->getApp(appid, app)) {
        showMenu(MenuProvider::DataType, QVariant::fromValue(app), location, lid, point);
    }
}

void MenuManager::showMenu(const DataEntity &entity, const MenuInfo::Location location, const QString &lid,
                           const QPoint &point)
{
    showMenu(MenuProvider::DataType, QVariant::fromValue(entity), location, lid, point);
}

void MenuManager::showMenu(const MenuProvider::RequestType &type, QVariant data, const MenuInfo::Location location, const QString &lid, const QPoint &point)
{
    QMenu menu;

    QList<QAction*> actions;
    for (const auto &provider : d->providers) {
        if (provider->isSupport(type)) {
            actions.append(provider->generateActions(&menu, data, location, lid));
        }
    }

    if (actions.isEmpty() && menu.isEmpty()) {
        return;
    }

    menu.addActions(actions);
    menu.exec(checkPoint(point));
}

inline QPoint MenuManager::checkPoint(const QPoint &rawPoint)
{
    if (rawPoint.isNull()) {
        return QCursor::pos();
    }
    return rawPoint;
}

void MenuManager::loadMenus()
{
    QDir pluginsDir(UKUI_MENU_CONTEXT_MENU_DIR);
    pluginsDir.setFilter(QDir::Files);
    for(const QString& fileName : pluginsDir.entryList(QDir::Files)) {
        QPluginLoader pluginLoader(pluginsDir.absoluteFilePath(fileName));
        QJsonObject metaData = pluginLoader.metaData().value("MetaData").toObject();
        QString type = metaData.value("Type").toString();
        QString version = metaData.value("Version").toString();
        if(type != UKUI_MENU_CONTEXTMENU_TYPE) {
            continue;
        }

        if(version != UKUI_MENU_CONTEXTMENU_IFACE_VERSION) {
            qWarning() << "UKUI_MENU_CONTEXTMENU version check failed:" << fileName << "version:" << version << "iface version : " << UKUI_MENU_CONTEXTMENU_IFACE_VERSION;
            continue;
        }

        QObject *obj = pluginLoader.instance();
        if (!obj) {
            continue;
        }

        MenuProvider *provider = qobject_cast<MenuProvider*>(obj);
        if (!provider) {
            continue;
        }

        registerMenuProvider(provider);
    }
}
