/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "settings.h"

#include <QVariant>
#include <QDebug>
#include <QtQml>

#define UKUI_MENU_SCHEMA                "org.ukui.menu.settings"
#define CONTROL_CENTER_SETTING          "org.ukui.control-center.personalise"
#define CONTROL_CENTER_TRANSPARENCY_KEY "transparency"
#define CONTROL_CENTER_EFFECT_KEY       "effect"

#define UKUI_STYLE_SCHEMA               "org.ukui.style"
#define UKUI_STYLE_NAME_KEY             "styleName"
#define UKUI_STYLE_THEME_COLOR_KEY      "themeColor"

namespace UkuiMenu {

void SettingModule::defineModule(const char *uri, int versionMajor, int versionMinor)
{
    qRegisterMetaType<GlobalSetting::Key>("GlobalSetting::Key");
    qmlRegisterUncreatableType<GlobalSetting>(uri, versionMajor, versionMinor, "GlobalSetting", "Use enums only.");
}

GlobalSetting *GlobalSetting::instance()
{
    static GlobalSetting setting(nullptr);
    return &setting;
}

GlobalSetting::GlobalSetting(QObject *parent) : QObject(parent)
{
    initStyleSetting();
}

void GlobalSetting::initStyleSetting()
{
    m_cache.insert(StyleName, UKUI_STYLE_VALUE_LIGHT);
    m_cache.insert(ThemeColor, UKUI_STYLE_THEME_COLOR_KEY);
    m_cache.insert(Transparency, 1);
    m_cache.insert(EffectEnabled, false);

    if (QGSettings::isSchemaInstalled(UKUI_STYLE_SCHEMA)) {
        QGSettings *settings = new QGSettings(UKUI_STYLE_SCHEMA, {}, this);

        QStringList keys = settings->keys();
        if (keys.contains(UKUI_STYLE_NAME_KEY)) {
            m_cache.insert(StyleName, settings->get(UKUI_STYLE_NAME_KEY));
        }
        if (keys.contains(UKUI_STYLE_THEME_COLOR_KEY)) {
            m_cache.insert(ThemeColor,settings->get(UKUI_STYLE_THEME_COLOR_KEY));
        }

        connect(settings, &QGSettings::changed, this, [this, settings] (const QString &key) {
            if (key == UKUI_STYLE_NAME_KEY) {
                updateData(StyleName, settings->get(key));
                Q_EMIT styleChanged(StyleName);
            } else if (key == UKUI_STYLE_THEME_COLOR_KEY) {
                updateData(ThemeColor, settings->get(key));
                Q_EMIT styleChanged(ThemeColor);
            }
        });
    }

    if (QGSettings::isSchemaInstalled(CONTROL_CENTER_SETTING)) {
        QGSettings *settings = new QGSettings(CONTROL_CENTER_SETTING,{},this);

        QStringList keys = settings->keys();
        if (keys.contains(CONTROL_CENTER_TRANSPARENCY_KEY)) {
            m_cache.insert(Transparency, settings->get(CONTROL_CENTER_TRANSPARENCY_KEY).toReal());
        }
        if (keys.contains(CONTROL_CENTER_EFFECT_KEY)) {
            m_cache.insert(EffectEnabled, settings->get(CONTROL_CENTER_EFFECT_KEY).toBool());
        }

        connect(settings, &QGSettings::changed, this, [this, settings] (const QString &key) {
            if (key == CONTROL_CENTER_TRANSPARENCY_KEY) {
                updateData(Transparency, settings->get(key).toReal());
                Q_EMIT styleChanged(Transparency);
            }
            if (key == CONTROL_CENTER_EFFECT_KEY) {
                updateData(EffectEnabled, settings->get(key).toReal());
                Q_EMIT styleChanged(EffectEnabled);
            }
        });
    }
}

QVariant GlobalSetting::get(const GlobalSetting::Key& key)
{
    QMutexLocker locker(&mutex);
    return m_cache.value(key);
}

void GlobalSetting::updateData(const GlobalSetting::Key &key, const QVariant &value)
{
    QMutexLocker locker(&mutex);
    m_cache.insert(key, value);
}

MenuSetting *MenuSetting::instance()
{
    static MenuSetting setting(nullptr);
    return &setting;
}

MenuSetting::MenuSetting(QObject *parent) : QObject(parent)
{
    initSettings();
}

void MenuSetting::initSettings()
{
    m_cache.insert(UKUI_MENU_ANIMATION_DURATION, {300});
    m_cache.insert(FOLLOW_UKUI_PANEL, {true});
    m_cache.insert(MENU_WIDTH, {350});
    m_cache.insert(MENU_HEIGHT, {540});
    m_cache.insert(MENU_MARGIN, {8});

    QByteArray id{UKUI_MENU_SCHEMA};
    if (QGSettings::isSchemaInstalled(id)) {
        m_gSettings = new QGSettings(id, "/", this);

        if (!m_gSettings) {
            qWarning() << "Unable to initialize settings.";
            return;
        }

        for (const auto &key: m_gSettings->keys()) {
            m_cache.insert(key, m_gSettings->get(key));
        }

        connect(m_gSettings, &QGSettings::changed, this, &MenuSetting::updateValue);
    }
}

QVariant MenuSetting::get(const QString &key)
{
    QMutexLocker locker(&mutex);
    return m_cache.value(key, {});
}

bool MenuSetting::set(const QString &key, const QVariant &value)
{
    QMutexLocker locker(&mutex);
    if (m_gSettings) {
        m_gSettings->set(key, value);
//        m_cache.insert(key,value);
//        Q_EMIT changed(key);
        return true;
    }

    return false;
}

void MenuSetting::updateValue(const QString &key)
{
    mutex.lock();
    m_cache.insert(key, m_gSettings->get(key));
    mutex.unlock();
    Q_EMIT changed(key);
}

} // UkuiMenu
