/*
 * Copyright (C) 2022, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * Authors: hxf <hewenfei@kylinos.cn>
 *
 */

#include "theme-palette.h"

#include <QGuiApplication>
#include <QColor>
#include <QPalette>

using namespace UkuiMenu;

static ThemePalette *globalInstance = nullptr;

ThemePalette *ThemePalette::getInstance()
{
    if (!globalInstance) {
        globalInstance = new ThemePalette(nullptr);
    }
    return globalInstance;
}

ThemePalette::ThemePalette(QObject *parent) : QObject(parent)
{
    initStyleSetting();
    connect(qGuiApp, &QGuiApplication::paletteChanged, this, [=] {
        Q_EMIT styleColorChanged();
    });
}

QColor ThemePalette::paletteColor(Palette::ColorRole colorRole, Palette::ColorGroup colorGroup) const
{
    switch (colorRole) {
        default:
        case Palette::Window:
            return window(colorGroup);
        case Palette::WindowText:
            return windowText(colorGroup);
        case Palette::Base:
            return base(colorGroup);
        case Palette::Text:
            return text(colorGroup);
        case Palette::AlternateBase:
            return alternateBase(colorGroup);
        case Palette::Button:
            return button(colorGroup);
        case Palette::ButtonText:
            return buttonText(colorGroup);
        case Palette::Light:
            return light(colorGroup);
        case Palette::MidLight:
            return midLight(colorGroup);
        case Palette::Dark:
            return dark(colorGroup);
        case Palette::Mid:
            return mid(colorGroup);
        case Palette::Shadow:
            return shadow(colorGroup);
        case Palette::Highlight:
            return highlight(colorGroup);
        case Palette::HighlightedText:
            return highlightedText(colorGroup);
    }
}

QColor
ThemePalette::paletteColorWithCustomTransparency(Palette::ColorRole colorRole, Palette::ColorGroup colorGroup,
                                                 qreal alphaF) const
{
    QColor color = paletteColor(colorRole, colorGroup);
    color.setAlphaF(alphaF);
    return color;
}

QColor
ThemePalette::paletteColorWithTransparency(Palette::ColorRole colorRole, Palette::ColorGroup colorGroup) const
{
    QColor color = paletteColor(colorRole, colorGroup);
    color.setAlphaF(m_transparency);
    return color;
}

QColor ThemePalette::window(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Window);
}

QColor ThemePalette::windowText(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::WindowText);
}

QColor ThemePalette::text(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Text);
}

QColor ThemePalette::base(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Base);
}

QColor ThemePalette::alternateBase(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::AlternateBase);
}

QColor ThemePalette::button(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Button);
}

QColor ThemePalette::buttonText(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::ButtonText);
}

QColor ThemePalette::light(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Light);
}

QColor ThemePalette::midLight(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Midlight);
}

QColor ThemePalette::dark(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Dark);
}

QColor ThemePalette::mid(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Mid);
}

QColor ThemePalette::shadow(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Shadow);
}

QColor ThemePalette::highlight(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Highlight);
}

QColor ThemePalette::highlightedText(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::HighlightedText);
}

QColor ThemePalette::separator(Palette::ColorGroup colorGroup) const
{
    return QGuiApplication::palette().color(switchColorGroup(colorGroup), QPalette::Window);
}

QPalette::ColorGroup ThemePalette::switchColorGroup(Palette::ColorGroup colorGroup)
{
    switch (colorGroup) {
        default:
        case Palette::Active:
            return QPalette::Active;
        case Palette::Disabled:
            return QPalette::Disabled;
        case Palette::Inactive:
            return QPalette::Inactive;
    }
}

void ThemePalette::initStyleSetting()
{
    GlobalSetting *setting = GlobalSetting::instance();
    m_transparency = setting->get(GlobalSetting::Transparency).toReal();
    connect(setting, &GlobalSetting::styleChanged, this, &ThemePalette::styleChangedSlot);
}

void ThemePalette::styleChangedSlot(const GlobalSetting::Key& key)
{
    if (key & GlobalSetting::Transparency) {
        m_transparency = GlobalSetting::instance()->get(GlobalSetting::Transparency).toReal();
    }

    Q_EMIT styleColorChanged();
}
