/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */
#include <gio/gio.h>

#include "app-manager.h"
#include "app-data-manager.h"

#include <QDebug>
#include <QDBusReply>
#include <QDBusInterface>
#include <QProcess>

#define KYLIN_APP_MANAGER_NAME         "com.kylin.AppManager"
#define KYLIN_APP_MANAGER_PATH         "/com/kylin/AppManager"
#define KYLIN_APP_MANAGER_INTERFACE    "com.kylin.AppManager"

namespace UkuiMenu {
AppManager *AppManager::instance()
{
    static AppManager instance(nullptr);
    return &instance;
}

AppManager::AppManager(QObject *parent) : QObject(parent)
{
    m_appManagerDbusInterface = new QDBusInterface(KYLIN_APP_MANAGER_NAME,
                                                   KYLIN_APP_MANAGER_PATH,
                                                   KYLIN_APP_MANAGER_INTERFACE,
                                                   QDBusConnection::sessionBus());
    if (!m_appManagerDbusInterface) {
        qWarning() << "appManager dbus does not exists.";
    }
}

bool AppManager::launchApp(const QString &desktopFilePath)
{
    Q_EMIT request(UkuiMenuApplication::Hide);
    if (launchAppWithDBus(desktopFilePath)) {
        return true;
    }

    QString cmd = parseDesktopFile(desktopFilePath);
//    qDebug() << "launchApp:" << desktopFilePath << cmd;
    if (cmd.isEmpty()) {
        return false;
    }

    return QProcess::startDetached(cmd);
}

bool AppManager::launchBinaryApp(const QString &app, const QString &args)
{
    if (app.isEmpty()) {
        return false;
    }

    QString cmd = app + " " + args;
//    qDebug() << "launchBinaryApp" << cmd;
    if (launchAppWithDBus(cmd)) {
        return true;
    }

    return QProcess::startDetached(cmd);
}

void AppManager::appLaunched(const QString &desktopFilePath)
{
    AppDataManager::instance()->appLaunch(desktopFilePath);
}

bool AppManager::launchAppWithDBus(const QString &app)
{
    if (!m_appManagerDbusInterface) {
        return false;
    }

    QDBusReply<bool> status = m_appManagerDbusInterface->call("LaunchApp", app);
    return status.value();
}

QString AppManager::parseDesktopFile(const QString &desktopFilePath)
{
    // TODO: try QSettings?
    GKeyFile *keyFile = g_key_file_new();
    gboolean res = g_key_file_load_from_file(keyFile, desktopFilePath.toUtf8(), G_KEY_FILE_NONE, nullptr);
    if (!res) {
        return "";
    }

    gchar *exec = g_key_file_get_value(keyFile, "Desktop Entry", "Exec", nullptr);
    if (!exec) {
        g_key_file_free(keyFile);
        return "";
    }

    QStringList stringList = QString(exec).split(" ");

    g_free(exec);
    g_key_file_free(keyFile);

    if (stringList.isEmpty()) {
        return "";
    }

    QString program = stringList.first();
    stringList.removeFirst();

    // TODO: freedesktop group
    for (const auto &str : stringList) {
        if (!str.startsWith("%")) {
            program.append(" " + str);
        }
    }

    return program;
}

} // UkuiMenu
