/*
 * Copyright (C) 2023, KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include <QProcess>
#include <QQmlEngine>
#include <QMenu>
#include <QDBusReply>

#include "power-button.h"
#include "app-manager.h"

#define KYLIN_POWER_NORMAL_PATH_MAJOR "qrc:///res/icon/pad_mainpower.svg"

using namespace UkuiMenu;

PowerButton::PowerButton(QObject *parent) : QObject(nullptr)
{

}

void PowerButton::defineModule(const char *uri, int versionMajor, int versionMinor)
{
    qmlRegisterType<PowerButton>("org.ukui.menu.utils", versionMajor, versionMinor, "PowerButton");
}

PowerButton::~PowerButton()
= default;

QString PowerButton::getIcon()
{
    return {KYLIN_POWER_NORMAL_PATH_MAJOR};
}

QString PowerButton::getToolTip()
{
    return {tr("power")};
}

void PowerButton::clicked(bool leftButtonClicked, int mouseX, int mouseY, bool isFullScreen)
{
        openMenu(mouseX, mouseY, isFullScreen);
}

void PowerButton::openMenu(int menuX, int menuY, bool isFullScreen)
{
    //<!1>增加变量如果已经显示，则不在显示 2023-05-29 add by fyl
    static bool _alreadyShow = false;
    if(_alreadyShow)
        return;
    //<!1>增加变量如果已经显示，则不在显示 2023-05-29 add by fyl
    QMenu powerMenu;
    QDBusReply<bool> reply;
    QDBusInterface qDBusInterface("org.gnome.SessionManager", "/org/gnome/SessionManager",
                                  "org.gnome.SessionManager", QDBusConnection::sessionBus());

    if (canSwitch() && hasMultipleUsers()) {
        connect(powerMenu.addAction(tr("Switch user")), &QAction::triggered, &powerMenu, [] {
               QDBusInterface *dbus = new QDBusInterface("org.gnome.SessionManager", "/org/gnome/SessionManager",
                            "org.gnome.SessionManager", QDBusConnection::sessionBus());
    if (!dbus->isValid()) {
        qWarning() << "dbusCall: QDBusInterface is invalid";
        return;
    }

    dbus->call("switchUser");
        });
    }

    reply = qDBusInterface.call("canHibernate");
    if (reply.isValid() && reply.value()) {
        connect(powerMenu.addAction(tr("Hibernate")), &QAction::triggered, &powerMenu, [] {
                QDBusInterface *dbus = new QDBusInterface("org.gnome.SessionManager", "/org/gnome/SessionManager",
                            "org.gnome.SessionManager", QDBusConnection::sessionBus());
    if (!dbus->isValid()) {
        qWarning() << "dbusCall: QDBusInterface is invalid";
        return;
    }

    dbus->call("hibernate");
        });
    }

    reply = qDBusInterface.call("canSuspend");
    if (reply.isValid() && reply.value()) {
        connect(powerMenu.addAction(tr("Suspend")), &QAction::triggered, &powerMenu, [] {
                QDBusInterface *dbus = new QDBusInterface("org.gnome.SessionManager", "/org/gnome/SessionManager",
                            "org.gnome.SessionManager", QDBusConnection::sessionBus());
    if (!dbus->isValid()) {
        qWarning() << "dbusCall: QDBusInterface is invalid";
        return;
    }

    dbus->call("suspend");
        });
    }

    connect(powerMenu.addAction(tr("Lock Screen")), &QAction::triggered, &powerMenu, [] {
        AppManager::instance()->launchBinaryApp("ukui-screensaver-command", "-l");
    });

    reply = qDBusInterface.call("canLogout");
    if (reply.isValid() && reply.value()) {
        connect(powerMenu.addAction(tr("Log Out")), &QAction::triggered, &powerMenu, [] {
                QDBusInterface *dbus = new QDBusInterface("org.gnome.SessionManager", "/org/gnome/SessionManager",
                            "org.gnome.SessionManager", QDBusConnection::sessionBus());
    if (!dbus->isValid()) {
        qWarning() << "dbusCall: QDBusInterface is invalid";
        return;
    }

    dbus->call("logout");
        });
    }

    reply = qDBusInterface.call("canReboot");
    if (reply.isValid() && reply.value()) {
        connect(powerMenu.addAction(tr("Reboot")), &QAction::triggered, &powerMenu, [] {
                QDBusInterface *dbus = new QDBusInterface("org.gnome.SessionManager", "/org/gnome/SessionManager",
                            "org.gnome.SessionManager", QDBusConnection::sessionBus());
    if (!dbus->isValid()) {
        qWarning() << "dbusCall: QDBusInterface is invalid";
        return;
    }

    dbus->call("reboot");
        });
    }

    reply = qDBusInterface.call("canPowerOff");
    if (reply.isValid() && reply.value()) {
        connect(powerMenu.addAction(tr("Power Off")), &QAction::triggered, &powerMenu, [] {
                QDBusInterface *dbus = new QDBusInterface("org.gnome.SessionManager", "/org/gnome/SessionManager",
                            "org.gnome.SessionManager", QDBusConnection::sessionBus());
    if (!dbus->isValid()) {
        qWarning() << "dbusCall: QDBusInterface is invalid";
        return;
    }

    dbus->call("powerOff");
        });
    }

    powerMenu.setAttribute(Qt::WA_DeleteOnClose);

    _alreadyShow = true;
    if (isFullScreen) {
        powerMenu.exec(QPoint(menuX - powerMenu.sizeHint().width(), menuY - powerMenu.sizeHint().height()));
    } else {
        powerMenu.exec(QPoint(menuX, menuY - powerMenu.sizeHint().height()));
    }
    _alreadyShow = false;
}

bool PowerButton::hasMultipleUsers()
{
    QDBusInterface interface("org.freedesktop.Accounts", "/org/freedesktop/Accounts",
                             "org.freedesktop.DBus.Properties", QDBusConnection::systemBus());

    if (!interface.isValid()) {
        return false;
    }

    QDBusReply<QVariant> reply = interface.call("Get", "org.freedesktop.Accounts", "HasMultipleUsers");
    return reply.value().toBool();
}

bool PowerButton::canSwitch()
{
    QDBusInterface interface("org.freedesktop.DisplayManager", "/org/freedesktop/DisplayManager/Seat0",
                             "org.freedesktop.DBus.Properties", QDBusConnection::systemBus());

    if (!interface.isValid()) {
        return false;
    }

    QDBusReply<QVariant> reply = interface.call("Get", "org.freedesktop.DisplayManager.Seat", "CanSwitch");
    return reply.value().toBool();
}
